use crate::prelude::*;

use std::os::raw::c_void;
use std::ptr;

#[repr(C)]
#[derive(Clone, Copy)]
pub union VkDeviceOrHostAddressConstKHR {
    deviceAddress: VkDeviceAddress,
    hostAddress: *const c_void,
}

impl VkDeviceOrHostAddressConstKHR {
    pub fn null() -> Self {
        VkDeviceOrHostAddressConstKHR {
            hostAddress: ptr::null(),
        }
    }

    pub fn device_address(&self) -> VkDeviceAddress {
        unsafe { self.deviceAddress }
    }

    pub fn host_address<T>(&self) -> &T {
        unsafe { &*(self.hostAddress as *const T) }
    }
}

impl From<VkDeviceAddress> for VkDeviceOrHostAddressConstKHR {
    fn from(device_address: VkDeviceAddress) -> Self {
        VkDeviceOrHostAddressConstKHR {
            deviceAddress: device_address,
        }
    }
}

impl<'a, T> From<&'a T> for VkDeviceOrHostAddressConstKHR {
    fn from(host_address: &'a T) -> Self {
        VkDeviceOrHostAddressConstKHR {
            hostAddress: host_address as *const T as *const c_void,
        }
    }
}

impl<'a, T> From<&'a [T]> for VkDeviceOrHostAddressConstKHR {
    fn from(host_address: &'a [T]) -> Self {
        VkDeviceOrHostAddressConstKHR {
            hostAddress: host_address.as_ptr() as *const c_void,
        }
    }
}

impl From<VkDeviceOrHostAddressKHR> for VkDeviceOrHostAddressConstKHR {
    fn from(address: VkDeviceOrHostAddressKHR) -> Self {
        VkDeviceOrHostAddressConstKHR {
            deviceAddress: address.device_address(),
        }
    }
}
